% Checks the schedulability of the taskset locally scheduled by {GEDF, FP}
% over the GMPR interface candidate.
% If schedulability is preserved, then the candidate is
% a valid GMPR interface.
% Schedulability test is Bini2009.
function isSchedulable = check_gmpr_candidate_Bini2009(gmprInterface, taskSet, schedPolicy)

Pi = gmprInterface.Pi;
thetas = gmprInterface.thetas;

n = numel(taskSet);
m = numel(thetas);


for i = 1:n
    
    isSchedulable = 0;
    
    for k = 1:m
        
        Ci = taskSet(i).C;
        Di = taskSet(i).D;
        
        if (strcmp(schedPolicy,'GEDF'))
            Wi = taskSet(i).Wedf;
        elseif (strcmp(schedPolicy,'FP'))
            Wi = taskSet(i).Wfp;
        else
            disp('Unknown scheduling policy, cannot check schedulability');
            return;
        end
        
        % the worst-case supply
        sbf_k_Di = gmpr_psf_k(k, thetas, Di, Pi);
        
        if (k*Ci + Wi <= sbf_k_Di)
            isSchedulable = 1;
            break;
        end
    end
    
    if (isSchedulable == 0)
        return;
    end
end

% all individual tasks are schedulable
% and overall schedulability is achieved
isSchedulable = 1;

end