% computes gmpr interface for a given taskset, Pi, m and a local scheduling
% policy {GEDF, FP}
function gmprInterfaces = compute_gmpr(Pi, m, taskSet, schedPolicy)

gmprInterfaces = struct('Pi', {}, 'thetas', {});

thetas = [];
searchSpace = get_thetas_search_space(taskSet, Pi, m, schedPolicy);
gmprInterfaces = generate_gmpr(1, m, taskSet, Pi, thetas, searchSpace, gmprInterfaces, schedPolicy);

end






% Recursively generates Thetas until a valid GMPR interface is found.
% At iteration i it searches Theta_i by enumerating values withing a
% search space
function gmprInterfaces = generate_gmpr(i, m, taskSet, Pi, thetas, searchSpace, gmprInterfaces, schedPolicy)

% check gmpr feasibility condition
if (numel(thetas) > 0)
    if (thetas(end) > (Pi*m))
        return;
    end
end

% Stop once m Thetas are generated, and
% check the schedulability condition under {GEDF, FP}
if (i > m)
    
    % Check if this set {Theta_k} is in the search space
%     disp('thetas'); disp(thetas);
     if (~are_thetas_in_search_space(thetas, searchSpace))
%         disp('thetas not in search space');
         return;
     end
    
    newInterface.Pi = Pi;
    newInterface.thetas = thetas;
    
    % check the GMPR interface using Bini2009
    if check_gmpr_candidate_Bini2009(newInterface, taskSet, schedPolicy)
        % interface is generated
        gmprInterfaces(numel(gmprInterfaces) + 1) = newInterface;
        return;
    else
        return;
    end
end

if (i == 1)
    
    for k = 1:Pi
        thetas(1) = k;
        gmprInterfaces = generate_gmpr(i+1, m, taskSet, Pi, thetas, searchSpace, gmprInterfaces, schedPolicy);
    end
    
else
    theta_i_min = thetas(i-1) + 1;
    if (i == 2)
        theta_i_max = min(Pi + thetas(1), (2*thetas(1)));
    else
        theta_i_max = min(Pi + thetas(i-1), 2*thetas(i-1)-thetas(i-2));
    end
    
    for k = theta_i_min:theta_i_max
        thetas(i) = k;
        gmprInterfaces = generate_gmpr(i+1, m, taskSet, Pi, thetas, searchSpace, gmprInterfaces, schedPolicy);
    end
end

end