% Computes MPR interface for a given taskset, Pi, m and scheduling policy
% {EDF, FP}.
% If there are no feasible interfaces for given Pi and m, then the list of
% MPR interfaces is empty.
function mprInterfaces = compute_mpr(Pi, m, taskSet, schedPolicy)

mprInterfaces = struct('Pi', {}, 'theta', {}, 'm', {});
searchSpace = get_thetas_search_space(taskSet, Pi, m, schedPolicy);

for theta = ((m-1)*Pi + 1):(m*Pi)
    
    % Consider the HV transformation
    thetas = get_hv_transformation(theta, m);
    
    % Check if the set {Theta_k} is in the search space
    if (~are_thetas_in_search_space(thetas, searchSpace))
        continue;
    end
    
    gmprInterface.Pi = Pi;
    gmprInterface.thetas = thetas;
    
    isSchedulable = check_gmpr_candidate_Bini2009(gmprInterface, taskSet, schedPolicy);
    
    if (isSchedulable == 1)
        
        newMprInterface.Pi = Pi;
        newMprInterface.theta = theta;
        newMprInterface.m = m;
        
        mprInterfaces(numel(mprInterfaces) + 1) = newMprInterface;
        return;
    end
end

end



% Gets the high-variance supply patern for the MPR interface
function thetas = get_hv_transformation(theta, m)

alpha = floor(theta/m);
beta = theta - alpha*m;
thetas = zeros(m,1);

for j = 1:beta
    if (j == 1)
        thetas(j) = alpha + 1;
    else
        thetas(j) = thetas(j-1) + alpha + 1;
    end
end

for j = (beta+1):m
    if (j == 1)
        thetas(j) = alpha;
    else
        thetas(j) = thetas(j-1) + alpha;
    end
end

end