% Generate task set having a total utilization U, an individual task
% utilization not exceeding Umax, and the ration Tmax/Tmin.
%
% The idea is to randomly distribute the utilization U between individual
% tasks, then to randomly generate the period T and compute C.
function tasks = generate_taskset(U, Umax, Tmaxmin)

tasks = struct('C', {}, 'T', {}, 'D', {}, 'Wedf', {}, 'Wfp', {}, 'P', {});


%n = 1 + randi(4);
Tmin = 20 + randi(20);
Tmax = ceil(Tmin*Tmaxmin);

%tasks = generate_CTD(U, Umax, Tmin, Tmax, tasks, n);
tasks = generate_CTD1(U, Umax, Tmin, Tmax, tasks);
tasks = get_interfering_workloads(tasks);

end





function tasks = generate_CTD(U, Umax, Tmin, Tmax, tasks, n)

Ur = U;

for i = 1:(n - 1)
    Ui = 1/Tmax + rand(1)*(min(Umax, Ur - (n-i+1)/Tmax) - 1/Tmax);
    Ur = Ur - Ui;
    tasks(i).T = Tmin + randi(Tmax - Tmin);
    tasks(i).D = tasks(i).T;
    tasks(i).C = Ui*tasks(i).T;
end

Usum = 0;
for i = 1:(n-1)
    Usum = Usum + tasks(i).C/tasks(i).T;
end

Ulast = U - Usum;
if (Ulast <= Umax)
    tasks(n).T = Tmin + randi(Tmax-Tmin);
    tasks(n).D = tasks(n).T;
    tasks(n).C = Ulast*tasks(n).T;
    return;
else
    last_tasks = ceil(Ulast/Umax);
    for j = 1:last_tasks
        tasks(n-1+j).T = Tmin + randi(Tmax-Tmin);
        tasks(n-1+j).D = tasks(n-1+j).T;
        tasks(n-1+j).C = (Ulast/last_tasks)*tasks(n-1+j).T;
    end
end

end




function tasks = generate_CTD1(U, Umax, Tmin, Tmax, tasks)

Ur = U;
i = 1;

while (Ur > Umax)
    Ui = rand(1)*Umax;
    tasks(i).T = Tmin + randi(Tmax - Tmin);
    tasks(i).D = tasks(i).T;
    tasks(i).C = Ui*tasks(i).T;
    
    Ur = Ur - Ui;
    i = i + 1;
end

% the last task
tasks(i).T = Tmin + randi(Tmax - Tmin);
tasks(i).D = tasks(i).T;
tasks(i).C = Ur*tasks(i).T;
    

% for i = 1:(n - 1)
%     Ui = 1/Tmax + rand(1)*(min(Umax, Ur - (n-i+1)/Tmax) - 1/Tmax);
%     Ur = Ur - Ui;
%     tasks(i).T = Tmin + randi(Tmax - Tmin);
%     tasks(i).D = tasks(i).T;
%     tasks(i).C = Ui*tasks(i).T;
% end
% 
% Usum = 0;
% for i = 1:(n-1)
%     Usum = Usum + tasks(i).C/tasks(i).T;
% end
% 
% Ulast = U - Usum;
% if (Ulast <= Umax)
%     tasks(n).T = Tmin + randi(Tmax-Tmin);
%     tasks(n).D = tasks(n).T;
%     tasks(n).C = Ulast*tasks(n).T;
%     return;
% else
%     last_tasks = ceil(Ulast/Umax);
%     for j = 1:last_tasks
%         tasks(n-1+j).T = Tmin + randi(Tmax-Tmin);
%         tasks(n-1+j).D = tasks(n-1+j).T;
%         tasks(n-1+j).C = (Ulast/last_tasks)*tasks(n-1+j).T;
%     end
% end

end