% computes a lower linear bound for a search space of thetas
function S_thetas = get_thetas_search_space(taskSet, Pi, m, schedPolicy)

S_thetas = {};

for i = 1:numel(taskSet)
    
    vi = zeros(1,m);
    
    Ci = taskSet(i).C;
    Di = taskSet(i).D;
    
    if (strcmp(schedPolicy,'GEDF'))
        Wi = taskSet(i).Wedf;
    elseif (strcmp(schedPolicy,'FP'))
        Wi = taskSet(i).Wfp;
    else
        disp('Unknown scheduling policy: Cannot compute search space for thetas');
        return;
    end
    
    % get vector
    for k = 1:m
        vi(k) = ceil((Pi/Di)*(k*Ci + Wi));
    end
    
    % add vector to the search space bound
    S_thetas = add_vector_to_search_space(S_thetas, vi, m);
end

end







% adds a new vector of thetas to the lower bound on the search space
function S_thetas = add_vector_to_search_space(S_thetas, v_i, m)

if isempty(S_thetas)
    S_thetas{1} = v_i;
    return;
end


% check if the vector should be ignored
% as an existing lower bound is already stricter
if (ignore_vector(v_i, S_thetas, m))
    return;
end

for j = 1:length(S_thetas)
    
    if (j > length(S_thetas))
        break;
    end
        
    v_j = S_thetas{j};
    v_i_dominates_v_j = 1;
    
    for k = 1:m
        if (v_j(k) > v_i(k))
            v_i_dominates_v_j = 0;
            break;
        end
    end
    
    if (v_i_dominates_v_j == 1)
        % remove v_j from S_thetas
        S_new_thetas(1:j-1) = S_thetas(1:j-1);
        S_new_thetas(j:length(S_thetas)-1) = S_thetas(j+1:end);
        S_thetas = S_new_thetas;
    end
end

% add v_i to S_thetas
S_thetas{end+1} = v_i;

end




% check if the vector should be ignored
% as an existing lower bound is already stricter
function ignore = ignore_vector(v_i, S_thetas, m)

ignore = 0;

for j = 1:length(S_thetas)
    
    v_j = S_thetas{j};
    dominates = 1;
    
    for k = 1:m
        if (v_i(k) > v_j(k))
            dominates = 0;
            break;
        end
    end
    
    if (dominates)
        ignore = 1;
        return;
    end
end

end